use std::collections::HashMap;
use hyper::Method;
use regex::RegexSet;

use super::regex::OwnedCaptures;
use super::route::Route;

macro_rules! try_opt {
    ($e:expr) => {
        match $e {
            Some(v) => v,
            None => return None,
        }
    };
}


#[derive(Default, Clone)]
pub struct Router {
    routes: HashMap<Method, Vec<Route>>,
}

impl Router {
    pub(crate) fn add_route(&mut self, route: Route) -> &mut Self {
        self.routes
            .entry(route.method.clone())
            .or_insert(Vec::new())
            .push(route);
        self
    }

    pub fn recognize(&self, method: &Method, path: &str) -> Option<(&Route, OwnedCaptures)> {
        let route = try_opt!(self.find_route(method, path));
        let caps = try_opt!(route.pattern.owned_captures(path));
        Some((route, caps))
    }

    // TODO: ranking
    pub fn find_route(&self, method: &Method, path: &str) -> Option<&Route> {
        let routes = try_opt!(self.routes.get(method));
        let set = RegexSet::new(routes.iter().map(|r| r.pattern_str())).unwrap();
        let matched = try_opt!(set.matches(path).into_iter().next());
        Some(&routes[matched])
    }
}
